==> skypager-project-2.47.0.tgz: package/lib/helpers/action.js <==
'use strict';

var _defineProperties = require('babel-runtime/core-js/object/define-properties');

var _defineProperties2 = _interopRequireDefault(_defineProperties);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _helper = require('./helper');

var _helper2 = _interopRequireDefault(_helper);

var _registry = require('../registry');

var _registry2 = _interopRequireDefault(_registry);

var _action = require('./definitions/action');

var _util = require('../util');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Action = (function (_Helper) {
  (0, _inherits3.default)(Action, _Helper);

  function Action() {
    (0, _classCallCheck3.default)(this, Action);
    return (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(Action).apply(this, arguments));
  }

  (0, _createClass3.default)(Action, [{
    key: 'attachCLI',

    /**
     * Configures a CLI utility such as commander to be able to expose
     * this action over a CLI interface.  Any additional context passed
     * from the CLI will be included, and we will make sure the project
     * is set
    */
    value: function attachCLI(commander, dispatchFn) {
      this.project && this.project.warn('using deprecated style of cli actions');

      var builder = this.get('definition.interfaces.cli');

      if (!builder) {
        throw 'Can not attach a CLI interface to this action. Missing definition CLI handler';
      }

      if (commander && (0, _typeof3.default)(commander.action === 'function') && typeof commander.command === 'function' && (0, _typeof3.default)(commander.option === 'function')) {
        builder.call(this, commander, dispatchFn);
      }
    }

    /**
     * This creates a promise wrapper around the
     * legacy action definition runner pipeline
    */

  }, {
    key: 'dispatch',
    value: function dispatch() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var action = this;
      var _context = args[args.length - 1];
      var project = this.hostProject;

      // getting called through the registry
      if (_context && _context.project && _context.__registryType) {
        project = _context.project;
      }

      var context = (0, _extends3.default)({}, _context, {
        get action() {
          return action;
        },
        get currentHelper() {
          return action;
        },
        project: project
      });

      var report = context.report = context.report || new Report(this, { console: false, project: project, args: args });

      args.push(context);

      return action.sanityCheck.apply(action, args).then(function () {
        var _action$validateWithG;

        return (_action$validateWithG = action.validateWithGlobalHelpers).call.apply(_action$validateWithG, [action].concat(args));
      }).then(function () {
        var _action$executeWithGl;

        return (_action$executeWithGl = action.executeWithGlobalHelpers).call.apply(_action$executeWithGl, [action].concat(args));
      }).then(function () {
        var context = args[args.length - 1];
        var report = context.report;

        if (!report._done) {
          report.done.apply(report, arguments);
        }

        return report;
      });
    }

    /**
     * @private
     *
     * This will get called at the beginning of the action run pipeline
     *
     * It will ensure there is a report, project, and a helpers object
     * on the final context argument.
     */

  }, {
    key: 'sanityCheck',
    value: function sanityCheck() {
      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      var action = this;
      var context = args[args.length - 1];

      if (!context || (typeof context === 'undefined' ? 'undefined' : (0, _typeof3.default)(context)) !== 'object') {
        throw 'Can not call action without the context object';
      }

      if (!context.report && !context.project) {
        throw 'Can not call action without report and project context';
      }

      var project = context.project;
      var report = context.report;

      context.helpers = {
        error: report.error.bind(report),
        warn: report.warn.bind(report),
        info: report.info.bind(report),
        suggest: report.suggest.bind(report),
        done: report.done.bind(report),
        action: action,
        report: report,
        project: project
      };

      return new _bluebird2.default(function (resolve, reject) {
        if ((typeof context === 'undefined' ? 'undefined' : (0, _typeof3.default)(context)) === 'object' && (0, _typeof3.default)(context.project) === 'object' && (0, _typeof3.default)(context.report) === 'object') {
          resolve.apply(undefined, args);
        } else {
          console.log(context);
          throw 'wtf bro';
        }
      });
    }
  }, {
    key: 'validateWithGlobalHelpers',
    value: function validateWithGlobalHelpers() {
      for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        args[_key3] = arguments[_key3];
      }

      var action = this;
      var context = args[args.length - 1];
      var project = context.project;
      var report = context.report;
      var helpers = context.helpers;

      var decorated = (0, _util.noConflict)(action.api.validate.bind(action), helpers);

      return new _bluebird2.default(function (resolve, reject) {
        try {
          var pass = decorated.apply(undefined, args);

          if (pass === false || report.errors.length > 0) {
            throw 'Action ' + action.id + ' failed validation step';
          } else {
            resolve.apply(undefined, args);
          }
        } catch (error) {
          report.error(error);
          reject(report);
        }
      });
    }
  }, {
    key: 'executeWithGlobalHelpers',
    value: function executeWithGlobalHelpers() {
      for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        args[_key4] = arguments[_key4];
      }

      var action = this;
      var context = args[args.length - 1];
      var project = context.project;
      var report = context.report;
      var helpers = context.helpers;

      var decorated = (0, _util.noConflict)(action.api.execute.bind(project), helpers);

      return new _bluebird2.default(function (resolve, reject) {
        try {
          var result = decorated.apply(undefined, args);
          resolve(result);
        } catch (error) {
          reject(error);
        }
      });
    }
  }, {
    key: 'reportError',
    value: function reportError(error) {
      this.project && this.project.error('error running action: ' + this.id, error);
      console.error('error running action: ' + this.id, error);
      return false;
    }
  }, {
    key: 'interfaces',
    get: function get() {
      return this.definition.interfaces;
    }
  }, {
    key: 'helperClass',
    get: function get() {
      return Action;
    }
  }, {
    key: 'definitionClass',
    get: function get() {
      return _action.ActionDefinition;
    }
  }, {
    key: 'host',
    get: function get() {
      return this.owner && this.owner.host;
    }
  }, {
    key: 'hostProject',
    get: function get() {
      return this.project || (this.host && this.host.type === 'project' ? this.host : undefined);
    }
  }], [{
    key: 'registryInterface',
    value: function registryInterface(registry) {
      return {
        actionsForCLI: {
          get: function get() {
            return registry.query(function (action) {
              return action.get('definition.interfaces.cli');
            });
          }
        },
        actionsForIPC: {
          get: function get() {
            return registry.query(function (action) {
              return action.get('definition.interfaces.ipc');
            });
          }
        },
        endpoints: {
          get: function get() {
            return registry.query(function (action) {
              return action.get('definition.interfaces.endpoint');
            });
          }
        }
      };
    }
  }]);
  return Action;
})(_helper2.default); /**
                       * REFACTOR NOTES
                       *
                       * 1) move the definition runner pipeline into this class.
                       *     definitions are just designed to provide a configuration DSL, not to determine behavior
                       *
                       * 2) the purpose of an action is to accept a request. the request should be structured as a
                       *    standard flux action.  the action is responsible for processing this action, but will first
                       *    validate and record the request before executing it.
                       *
                       * 3) the action system can handle both mutation requests, and query type requests. it is up to the
                       *    person writing the action.  the expectation is that each request will receive a standard type of
                       *    response.
                       *
                       * 4) actions can be exposed over several interfaces:
                       *      a) rest
                       *      b) cli (commander)
                       *      c) ipc
                       *      d) streams ? (wip)
                      */

Action.DSL = _action.DSL;
Action.Definition = _action.ActionDefinition;
Action.apiMethods = _action.ActionDefinition.apiMethods;

module.exports = Action;

var Report = (function () {
  function Report(action) {
    var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
    (0, _classCallCheck3.default)(this, Report);

    this.errors = [];
    this.notices = [];
    this.warnings = [];

    var project = options.project || action.hostProject;

    (0, _defineProperties2.default)(this, {
      options: { enumerable: false, value: options },
      _info: {
        enumerable: false,
        value: function value() {
          var logger = project ? project.info : console.log;
          logger.apply(undefined, arguments);
        }
      },
      _warn: {
        enumerable: false,
        value: function value() {
          var logger = project ? project.warn : console.warn;
          logger.apply(undefined, arguments);
        }
      },
      _error: {
        enumerable: false,
        value: function value() {
          var logger = project ? project.error : console.error;
          logger.apply(undefined, arguments);
        }
      },
      _suggest: {
        enumerable: false,
        value: function value() {
          var logger = project ? project.info : console.log;
          logger.apply(undefined, arguments);
        }
      }
    });
  }

  (0, _createClass3.default)(Report, [{
    key: 'log',
    value: function log(message) {
      for (var _len5 = arguments.length, args = Array(_len5 > 1 ? _len5 - 1 : 0), _key5 = 1; _key5 < _len5; _key5++) {
        args[_key5 - 1] = arguments[_key5];
      }

      this._info.apply(this, [message].concat(args));
    }
  }, {
    key: 'error',
    value: function error(message) {
      this.errors.push(message);

      for (var _len6 = arguments.length, args = Array(_len6 > 1 ? _len6 - 1 : 0), _key6 = 1; _key6 < _len6; _key6++) {
        args[_key6 - 1] = arguments[_key6];
      }

      this._error.apply(this, [message].concat(args));
    }
  }, {
    key: 'warn',
    value: function warn(message) {
      this.warnings.push(message);

      for (var _len7 = arguments.length, args = Array(_len7 > 1 ? _len7 - 1 : 0), _key7 = 1; _key7 < _len7; _key7++) {
        args[_key7 - 1] = arguments[_key7];
      }

      this._warn.apply(this, [message].concat(args));
    }
  }, {
    key: 'suggest',
    value: function suggest(message) {
      for (var _len8 = arguments.length, args = Array(_len8 > 1 ? _len8 - 1 : 0), _key8 = 1; _key8 < _len8; _key8++) {
        args[_key8 - 1] = arguments[_key8];
      }

      this._info.apply(this, [message].concat(args));
    }
  }, {
    key: 'info',
    value: function info(message) {
      this.notices.push(message);

      for (var _len9 = arguments.length, args = Array(_len9 > 1 ? _len9 - 1 : 0), _key9 = 1; _key9 < _len9; _key9++) {
        args[_key9 - 1] = arguments[_key9];
      }

      this._info.apply(this, [message].concat(args));
    }
  }, {
    key: 'done',
    value: function done(result) {
      this.result = result;
      this._done = true;
      this._info('successfully ran action');
      return this;
    }
  }, {
    key: 'success',
    get: function get() {
      return this.errors.length === 0;
    }
  }]);
  return Report;
})();

==> skypager-project-2.55.2.tgz: package/lib/helpers/action.js <==
'use strict';

var _defineProperties = require('babel-runtime/core-js/object/define-properties');

var _defineProperties2 = _interopRequireDefault(_defineProperties);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _helper = require('./helper');

var _helper2 = _interopRequireDefault(_helper);

var _registry = require('../registry');

var _registry2 = _interopRequireDefault(_registry);

var _action = require('./definitions/action');

var _util = require('../util');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Action = (function (_Helper) {
  (0, _inherits3.default)(Action, _Helper);

  function Action() {
    (0, _classCallCheck3.default)(this, Action);
    return (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(Action).apply(this, arguments));
  }

  (0, _createClass3.default)(Action, [{
    key: 'attachCLI',

    /**
     * Configures a CLI utility such as commander to be able to expose
     * this action over a CLI interface.  Any additional context passed
     * from the CLI will be included, and we will make sure the project
     * is set
    */
    value: function attachCLI(commander, dispatchFn) {
      this.project && this.project.warn('using deprecated style of cli actions');

      var builder = this.get('definition.interfaces.cli');

      if (!builder) {
        throw 'Can not attach a CLI interface to this action. Missing definition CLI handler';
      }

      if (commander && (0, _typeof3.default)(commander.action === 'function') && typeof commander.command === 'function' && (0, _typeof3.default)(commander.option === 'function')) {
        builder.call(this, commander, dispatchFn);
      }
    }

    /**
     * This creates a promise wrapper around the
     * legacy action definition runner pipeline
    */

  }, {
    key: 'dispatch',
    value: function dispatch() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var action = this;
      var _context = args[args.length - 1];
      var project = this.hostProject;

      // getting called through the registry
      if (_context && _context.project && _context.__registryType) {
        project = _context.project;
      }

      var context = (0, _extends3.default)({}, _context, {
        get action() {
          return action;
        },
        get currentHelper() {
          return action;
        },
        project: project
      });

      var report = context.report = context.report || new Report(this, { console: false, project: project, args: args });

      args.push(context);

      return action.sanityCheck.apply(action, args).then(function () {
        var _action$validateWithG;

        return (_action$validateWithG = action.validateWithGlobalHelpers).call.apply(_action$validateWithG, [action].concat(args));
      }).then(function () {
        var _action$executeWithGl;

        return (_action$executeWithGl = action.executeWithGlobalHelpers).call.apply(_action$executeWithGl, [action].concat(args));
      }).then(function () {
        var context = args[args.length - 1];
        var report = context.report;

        if (!report._done) {
          report.done.apply(report, arguments);
        }

        return report;
      });
    }

    /**
     * @private
     *
     * This will get called at the beginning of the action run pipeline
     *
     * It will ensure there is a report, project, and a helpers object
     * on the final context argument.
     */

  }, {
    key: 'sanityCheck',
    value: function sanityCheck() {
      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      var action = this;
      var context = args[args.length - 1];

      if (!context || (typeof context === 'undefined' ? 'undefined' : (0, _typeof3.default)(context)) !== 'object') {
        throw 'Can not call action without the context object';
      }

      if (!context.report && !context.project) {
        throw 'Can not call action without report and project context';
      }

      var project = context.project;
      var report = context.report;

      context.helpers = {
        error: report.error.bind(report),
        warn: report.warn.bind(report),
        info: report.info.bind(report),
        suggest: report.suggest.bind(report),
        done: report.done.bind(report),
        action: action,
        report: report,
        project: project
      };

      return new _bluebird2.default(function (resolve, reject) {
        if ((typeof context === 'undefined' ? 'undefined' : (0, _typeof3.default)(context)) === 'object' && (0, _typeof3.default)(context.project) === 'object' && (0, _typeof3.default)(context.report) === 'object') {
          resolve.apply(undefined, args);
        } else {
          console.log(context);
          throw 'wtf bro';
        }
      });
    }
  }, {
    key: 'validateWithGlobalHelpers',
    value: function validateWithGlobalHelpers() {
      for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        args[_key3] = arguments[_key3];
      }

      var action = this;
      var context = args[args.length - 1];
      var project = context.project;
      var report = context.report;
      var helpers = context.helpers;

      var decorated = (0, _util.noConflict)(action.api.validate.bind(action), helpers);

      var failedMessage = 'Action ' + action.id + ' failed validation step';
      return new _bluebird2.default(function (resolve, reject) {
        try {
          var pass = decorated.apply(undefined, args);
          var passed = _bluebird2.default.resolve(pass);

          passed.then(function (result) {
            if (result === false || report.errors.length > 0) {
              report.error(failedMessage);
              reject(report);
            } else {
              resolve.apply(undefined, args);
            }
          }).catch(function (err) {
            report.error(err);
            report.error(failedMessage);
            reject(report);
          });
        } catch (error) {
          report.error(error);
          report.error(failedMessage);
          reject(report);
        }
      });
    }
  }, {
    key: 'executeWithGlobalHelpers',
    value: function executeWithGlobalHelpers() {
      for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        args[_key4] = arguments[_key4];
      }

      var action = this;
      var context = args[args.length - 1];
      var project = context.project;
      var report = context.report;
      var helpers = context.helpers;

      var decorated = (0, _util.noConflict)(action.api.execute.bind(project), helpers);

      return new _bluebird2.default(function (resolve, reject) {
        try {
          var result = decorated.apply(undefined, args);
          resolve(result);
        } catch (error) {
          reject(error);
        }
      });
    }
  }, {
    key: 'reportError',
    value: function reportError(error) {
      this.project && this.project.error('error running action: ' + this.id, error);
      console.error('error running action: ' + this.id, error);
      return false;
    }
  }, {
    key: 'interfaces',
    get: function get() {
      return this.definition.interfaces;
    }
  }, {
    key: 'helperClass',
    get: function get() {
      return Action;
    }
  }, {
    key: 'definitionClass',
    get: function get() {
      return _action.ActionDefinition;
    }
  }, {
    key: 'host',
    get: function get() {
      return this.owner && this.owner.host;
    }
  }, {
    key: 'hostProject',
    get: function get() {
      return this.project || (this.host && this.host.type === 'project' ? this.host : undefined);
    }
  }], [{
    key: 'registryInterface',
    value: function registryInterface(registry) {
      return {
        actionsForCLI: {
          get: function get() {
            return registry.query(function (action) {
              return action.get('definition.interfaces.cli');
            });
          }
        },
        actionsForIPC: {
          get: function get() {
            return registry.query(function (action) {
              return action.get('definition.interfaces.ipc');
            });
          }
        },
        endpoints: {
          get: function get() {
            return registry.query(function (action) {
              return action.get('definition.interfaces.endpoint');
            });
          }
        }
      };
    }
  }]);
  return Action;
})(_helper2.default); /**
                       * REFACTOR NOTES
                       *
                       * 1) move the definition runner pipeline into this class.
                       *     definitions are just designed to provide a configuration DSL, not to determine behavior
                       *
                       * 2) the purpose of an action is to accept a request. the request should be structured as a
                       *    standard flux action.  the action is responsible for processing this action, but will first
                       *    validate and record the request before executing it.
                       *
                       * 3) the action system can handle both mutation requests, and query type requests. it is up to the
                       *    person writing the action.  the expectation is that each request will receive a standard type of
                       *    response.
                       *
                       * 4) actions can be exposed over several interfaces:
                       *      a) rest
                       *      b) cli (commander)
                       *      c) ipc
                       *      d) streams ? (wip)
                      */

Action.DSL = _action.DSL;
Action.Definition = _action.ActionDefinition;
Action.apiMethods = _action.ActionDefinition.apiMethods;

module.exports = Action;

var Report = (function () {
  function Report(action) {
    var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
    (0, _classCallCheck3.default)(this, Report);

    this.errors = [];
    this.notices = [];
    this.warnings = [];

    var project = options.project || action.hostProject;

    (0, _defineProperties2.default)(this, {
      options: { enumerable: false, value: options },
      _info: {
        enumerable: false,
        value: function value() {
          var logger = project ? project.info : console.log;
          logger.apply(undefined, arguments);
        }
      },
      _warn: {
        enumerable: false,
        value: function value() {
          var logger = project ? project.warn : console.warn;
          logger.apply(undefined, arguments);
        }
      },
      _error: {
        enumerable: false,
        value: function value() {
          var logger = project ? project.error : console.error;
          logger.apply(undefined, arguments);
        }
      },
      _suggest: {
        enumerable: false,
        value: function value() {
          var logger = project ? project.info : console.log;
          logger.apply(undefined, arguments);
        }
      }
    });
  }

  (0, _createClass3.default)(Report, [{
    key: 'log',
    value: function log(message) {
      for (var _len5 = arguments.length, args = Array(_len5 > 1 ? _len5 - 1 : 0), _key5 = 1; _key5 < _len5; _key5++) {
        args[_key5 - 1] = arguments[_key5];
      }

      this._info.apply(this, [message].concat(args));
    }
  }, {
    key: 'error',
    value: function error(message) {
      this.errors.push(message);

      for (var _len6 = arguments.length, args = Array(_len6 > 1 ? _len6 - 1 : 0), _key6 = 1; _key6 < _len6; _key6++) {
        args[_key6 - 1] = arguments[_key6];
      }

      this._error.apply(this, [message].concat(args));
    }
  }, {
    key: 'warn',
    value: function warn(message) {
      this.warnings.push(message);

      for (var _len7 = arguments.length, args = Array(_len7 > 1 ? _len7 - 1 : 0), _key7 = 1; _key7 < _len7; _key7++) {
        args[_key7 - 1] = arguments[_key7];
      }

      this._warn.apply(this, [message].concat(args));
    }
  }, {
    key: 'suggest',
    value: function suggest(message) {
      for (var _len8 = arguments.length, args = Array(_len8 > 1 ? _len8 - 1 : 0), _key8 = 1; _key8 < _len8; _key8++) {
        args[_key8 - 1] = arguments[_key8];
      }

      this._info.apply(this, [message].concat(args));
    }
  }, {
    key: 'info',
    value: function info(message) {
      this.notices.push(message);

      for (var _len9 = arguments.length, args = Array(_len9 > 1 ? _len9 - 1 : 0), _key9 = 1; _key9 < _len9; _key9++) {
        args[_key9 - 1] = arguments[_key9];
      }

      this._info.apply(this, [message].concat(args));
    }
  }, {
    key: 'done',
    value: function done(result) {
      this.result = result;
      this._done = true;
      this._info('successfully ran action');
      return this;
    }
  }, {
    key: 'success',
    get: function get() {
      return this.errors.length === 0;
    }
  }]);
  return Report;
})();
